# -*- coding: utf-8 -*-
"""
Created on Wed Feb 20 11:43:18 2019

@author: akrishnan
"""




def Avg(imp_param_up, bend_ang_up, imp_param_dn, bend_ang_dn,Bande):

    #imp_param = (imp_param_up + imp_param_dn)/2
    #bend_ang  = (bend_ang_up + bend_ang_dn)/2

    imp_param = imp_param_dn
    bend_ang  = bend_ang_dn
    #print('\tR9: Done')
    return imp_param,bend_ang
